@@ -12,8 +12,8 @@ Editra Buisness Model Library:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: __init__.py 60465 2009-05-02 00:16:48Z CJP $"
-__revision__ = "$Revision: 60465 $"
+__cvsid__ = "$Id: __init__.py 60840 2009-05-31 16:00:50Z CJP $"
+__revision__ = "$Revision: 60840 $"
 
 #-----------------------------------------------------------------------------#
 
@@ -22,3 +22,12 @@ from searcheng import *
 from fchecker import *
 from fileutil import *
 from fileimpl import *
+
+from backupmgr import *
+
+# Storage Classes
+from histcache import *
+from clipboard import *
+
+# Misc
+from miscutil import *
