@@ -35,8 +35,8 @@ eclib.ctrlbox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: finddlg.py 59519 2009-03-13 21:10:54Z CJP $"
-__revision__ = "$Revision: 59519 $"
+__svnid__ = "$Id: finddlg.py 60840 2009-05-31 16:00:50Z CJP $"
+__revision__ = "$Revision: 60840 $"
 
 __all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
            "MiniFindReplaceDlg", "AdvFindReplaceDlg",
@@ -49,13 +49,14 @@ __all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
            "AFR_NOUPDOWN", "AFR_NOWHOLEWORD", "AFR_NOMATCHCASE", "AFR_NOREGEX",
            "AFR_NOFILTER", "AFR_NOOPTIONS",
 
-           "LOCATION_CURRENT_DOC",
+           "LOCATION_CURRENT_DOC", "LOCATION_IN_SELECTION",
            "LOCATION_OPEN_DOCS", "LOCATION_IN_FILES", "LOCATION_MAX",
 
            "edEVT_FIND_CLOSE", "EVT_FIND_CLOSE", "edEVT_FIND", "EVT_FIND",
            "edEVT_FIND_NEXT", "EVT_FIND_NEXT", "edEVT_FIND_ALL", "EVT_FIND_ALL",
            "edEVT_REPLACE", "EVT_REPLACE", "edEVT_REPLACE_ALL",
-           "EVT_REPLACE_ALL", "EVT_OPTION_CHANGED", "edEVT_OPTION_CHANGED"]
+           "EVT_REPLACE_ALL", "EVT_OPTION_CHANGED", "edEVT_OPTION_CHANGED",
+           "EVT_COUNT", "edEVT_COUNT"]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -90,9 +91,10 @@ AFR_NOOPTIONS   = 2048          # Hide all options in the dialog
 
 # Search Location Parameters (NOTE: must be kept in sync with Lookin List)
 LOCATION_CURRENT_DOC  = 0
-LOCATION_OPEN_DOCS    = 1
-LOCATION_IN_FILES     = 2
-LOCATION_MAX          = 2
+LOCATION_IN_SELECTION = 1
+LOCATION_OPEN_DOCS    = 2
+LOCATION_IN_FILES     = 3
+LOCATION_MAX          = 3
 
 # Control Names
 FindBoxName = "EdFindBox"
@@ -106,6 +108,7 @@ ID_MATCH_CASE = wx.NewId()
 ID_WHOLE_WORD = wx.NewId()
 ID_REGEX = wx.NewId()
 ID_RECURSE = wx.NewId()
+ID_COUNT = wx.NewId()
 ID_FIND_ALL = wx.NewId()
 ID_REPLACE_ALL = wx.NewId()
 ID_OPTION_CHANGE = wx.NewId()
@@ -137,6 +140,10 @@ EVT_FIND_ALL = wx.PyEventBinder(edEVT_FIND_ALL, 1)
 edEVT_REPLACE = wx.NewEventType()
 EVT_REPLACE = wx.PyEventBinder(edEVT_REPLACE, 1)
 
+# Replace button clicked
+edEVT_COUNT = wx.NewEventType()
+EVT_COUNT = wx.PyEventBinder(edEVT_COUNT, 1)
+
 # Replace All button clicked
 edEVT_REPLACE_ALL = wx.NewEventType()
 EVT_REPLACE_ALL = wx.PyEventBinder(edEVT_REPLACE_ALL, 1)
@@ -148,6 +155,7 @@ EVT_OPTION_CHANGED = wx.PyEventBinder(edEVT_OPTION_CHANGED, 1)
 # Convenience for generating events
 _EVENT_MAP = { wx.ID_FIND : edEVT_FIND,
                wx.ID_REPLACE : edEVT_REPLACE,
+               ID_COUNT : edEVT_COUNT,
                ID_FIND_ALL : edEVT_FIND_ALL,
                ID_REPLACE_ALL : edEVT_REPLACE_ALL,
                ID_OPTION_CHANGE : edEVT_OPTION_CHANGED }
@@ -172,6 +180,14 @@ class FindEvent(wx.PyCommandEvent):
         self._replace = u''
         self._dir = u''
         self._filters = None
+        self._count = 0
+
+    def GetCount(self):
+        """Get the number of matches
+        @return: int
+
+        """
+        return self._count
 
     def GetDirectory(self):
         """Get the directory of files to search in
@@ -215,6 +231,13 @@ class FindEvent(wx.PyCommandEvent):
         """
         return self._loc
 
+    def SetCount(self, count):
+        """Set the count
+        @param count: int
+
+        """
+        self._count = count
+
     def SetDirectory(self, directory):
         """Set the directory of files to search in
         @param directory: string
@@ -320,7 +343,8 @@ def AdvFindReplaceDlg(parent, fdata, title, style=AFR_STYLE_FINDDIALOG):
 #--------------------------------------------------------------------------#
 
 class FindReplaceDlgBase:
-    """Mixin Base class for deriving FindReplaceDialogs.
+    """Delegate mixin base class for deriving FindReplaceDialogs, delegates
+    calls to the widgets and containers that make up the dialog.
     @note: The mixin must be initialized after the class its being mixed into
 
     """
@@ -458,7 +482,7 @@ class FindReplaceDlgBase:
         """
         self._box.SetFindBitmap(bmp)
 
-    def SetFlag(self, flag):
+    def SetFlag(self, flags):
         """Set a search dialog flag.
         @param flags: AFR_*
 
@@ -479,6 +503,15 @@ class FindReplaceDlgBase:
         """
         self._panel.SetLookinChoices(paths)
 
+    def SetLookinPath(self, path):
+        """Set the lookin path, adding it to the collection if it is
+        not in there.
+        @param path: string (path of directory)
+
+        """
+        idx = self._panel.AddLookinPath(path)
+        self.SetLookinSelection(idx)
+
     def SetLookinSelection(self, sel):
         """Set the looking choices selection
         @param sel: int
@@ -509,7 +542,7 @@ class FindReplaceDlgBase:
 
     def SetReplaceString(self, rstring):
         """Set the replace controls search string
-        @param query: string
+        @param rstring: string
 
         """
         self._panel.SetReplaceString(rstring)
@@ -661,7 +694,8 @@ class FindPanel(wx.Panel):
         self._mode = style
         self._ftxt = wx.TextCtrl(self, value=fdata.GetFindString())
         self._rtxt = wx.TextCtrl(self, value=fdata.GetReplaceString())
-        locations = [_("Current Document"), _("Open Documents")]
+        locations = [_("Current Document"), _("Selected Text"),
+                     _("Open Documents")]
         self._lookin = wx.Choice(self, ID_LOOKIN, choices=locations)
         self._lookin.SetSelection(0)
         self._filterlbl = wx.StaticText(self, label=_("File Filters:"))
@@ -686,7 +720,8 @@ class FindPanel(wx.Panel):
         self.Bind(wx.EVT_CHECKBOX, self.OnOption)
         self.Bind(wx.EVT_RADIOBUTTON, self.OnOption)
         self.Bind(wx.EVT_CHOICE, self.OnChoice, id=ID_LOOKIN)
-        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_FIND_ALL, ID_REPLACE_ALL):
+        for bid in (wx.ID_FIND, wx.ID_REPLACE, ID_COUNT,
+                    ID_FIND_ALL, ID_REPLACE_ALL):
             self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI, id=bid)
         self.Bind(wx.EVT_SET_FOCUS, lambda evt: self.__SetFocus())
         self._ftxt.Bind(wx.EVT_SET_FOCUS, lambda evt: self._ftxt.SelectAll())
@@ -792,6 +827,7 @@ class FindPanel(wx.Panel):
         self._sizers['frspacer'] = bsizer.Add((50, 1), 1)
         for bid, blbl in [(wx.ID_FIND, _("Find")),
                           (wx.ID_REPLACE, _("Replace")),
+                          (ID_COUNT, _("Count")),
                           (ID_FIND_ALL, _("Find All")),
                           (ID_REPLACE_ALL, _("Replace All"))]:
             self._sizers[bid] = wx.BoxSizer(wx.HORIZONTAL)
@@ -846,11 +882,11 @@ class FindPanel(wx.Panel):
 
         """
         if find:
-            show = (wx.ID_FIND, ID_FIND_ALL, 'fspacer')
+            show = (wx.ID_FIND, ID_COUNT, ID_FIND_ALL, 'fspacer')
             hide = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
         else:
             show = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
-            hide = (ID_FIND_ALL, 'fspacer')
+            hide = (ID_FIND_ALL, ID_COUNT, 'fspacer')
 
         for ctrl in show:
             if isinstance(ctrl, basestring):
@@ -886,18 +922,26 @@ class FindPanel(wx.Panel):
         find_all = self.FindWindowById(ID_FIND_ALL)
         replace = self.FindWindowById(wx.ID_REPLACE)
         replace_all = self.FindWindowById(ID_REPLACE_ALL)
+        count = self.FindWindowById(ID_COUNT)
 
-        if self._lookin.GetSelection() > LOCATION_CURRENT_DOC:
+        lookin = self._lookin.GetSelection()
+        if  lookin > LOCATION_IN_SELECTION:
             if self._mode == AFR_STYLE_FINDDIALOG:
                 find_all.SetDefault()
             else:
                 replace_all.SetDefault()
             find.Disable()
             replace.Disable()
+            count.Disable()
+        elif lookin == LOCATION_IN_SELECTION:
+            find.Disable()
+            replace.Disable()
+            replace_all.SetDefault()
         else:
             find.SetDefault()
             find.Enable()
             replace.Enable()
+            count.Enable()
 
     #------------------------------------------------------#
 
@@ -1077,7 +1121,8 @@ class FindPanel(wx.Panel):
         @param evt: wx.UpdateUIEvent
 
         """
-        if evt.GetId() in (wx.ID_FIND, wx.ID_REPLACE) and \
+        # TODO: support count in files?
+        if evt.GetId() in (wx.ID_FIND, ID_COUNT, wx.ID_REPLACE) and \
            self._lookin.GetSelection() > LOCATION_CURRENT_DOC:
             evt.Enable(False)
         else:
