@@ -18,8 +18,8 @@ a gradient using system defined colors.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_cmdbar.py 60041 2009-04-06 15:03:46Z CJP $"
-__revision__ = "$Revision: 60041 $"
+__svnid__ = "$Id: ed_cmdbar.py 61042 2009-06-13 08:21:56Z CJP $"
+__revision__ = "$Revision: 61042 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -35,6 +35,7 @@ import ed_glob
 import ed_search
 import ed_event
 import ed_msg
+import ebmlib
 import eclib
 
 _ = wx.GetTranslation
@@ -250,15 +251,17 @@ class CommandEntryBar(CommandBarBase):
 
         # Setup
         cmd_lbl = wx.StaticText(self, label=_("Command") + ": ")
+        self.info_lbl = wx.StaticText(self, label="")
         self.AddControl(cmd_lbl, wx.ALIGN_LEFT)
-        self.AddControl(self.ctrl, wx.ALIGN_LEFT)
+        self.AddControl(self.ctrl, 1, wx.ALIGN_LEFT)
+        self.AddControl(self.info_lbl, wx.ALIGN_RIGHT)
 
         # HACK: workaround bug in mac control that resets size to
         #       that of the default variant after any text has been
         #       typed in it. Note it reports the best size as the default
         #       variant and causes layout issues. wxBUG
         if wx.Platform == '__WXMAC__':
-            self.ctrl.SetSizeHints(150, 16, 150, 16)
+            self.ctrl.SetSizeHints(200, 16, -1, 16)
 
 #-----------------------------------------------------------------------------#
 
@@ -325,23 +328,27 @@ class CommandExecuter(eclib.CommandEntryBase):
     def _AdjustSize(self):
         """Checks width of text as its added and dynamically resizes
         the control as needed.
+        @todo: re-enable after resizing issue can be resolved
 
         """
-        ext = self.GetTextExtent(self.GetValue())[0]
-        curr_w, curr_h = self.GetClientSizeTuple()
-        if ext > curr_w * .5:
-            max_w = self.GetParent().GetClientSize().GetWidth() * .8
-            nwidth = min(ext * 1.3, max_w)
-            pwidth = self._popup.GetBestSize()[0]
-            if pwidth > nwidth:
-                nwidth = pwidth
-            self.SetClientSize((nwidth, curr_h))
-            self._popup.SetSize((nwidth, -1))
-        elif ((curr_w > ext * 1.18) and curr_w > 150):
-            nwidth = max(ext * 1.18, 150)
-            self.SetClientSize((nwidth, curr_h))
-        else:
-            pass
+        pass
+#        ext = self.GetTextExtent(self.GetValue())[0]
+#        curr_w, curr_h = self.GetClientSizeTuple()
+#        if ext > curr_w * .5:
+#            max_w = self.GetParent().GetClientSize().GetWidth() * .8
+#            nwidth = min(ext * 1.3, max_w)
+#            pwidth = self._popup.GetBestSize()[0]
+#            if pwidth > nwidth:
+#                nwidth = pwidth
+#            self.SetClientSize((nwidth, curr_h))
+#            self._popup.SetSize((nwidth, -1))
+#            self.GetParent().Layout()
+#        elif ((curr_w > ext * 1.18) and curr_w > 150):
+#            nwidth = max(ext * 1.18, 150)
+#            self.SetClientSize((nwidth, curr_h))
+#            self.GetParent().Layout()
+#        else:
+#            pass
 
     def _AdjustValue(self, val):
         """Adjust value of input string as autocomp provides new values
@@ -355,9 +362,10 @@ class CommandExecuter(eclib.CommandEntryBase):
             self.SetValue(" ".join([cval[0], val]))
         self.SetInsertionPoint(self.GetLastPosition())
 
-    def _UpdateCwd(self, msg=None):
+    def _UpdateCwd(self, msg):
         """Update the current working directory to that of the current
         buffer.
+        @param msg: Message Object
 
         """
         # Only Update if we are the currently active window
@@ -383,9 +391,18 @@ class CommandExecuter(eclib.CommandEntryBase):
                 path = os.path.join(self._curdir, path)
 
         if os.path.exists(path) and os.path.isdir(path):
-            os.chdir(path)
-            self._curdir = os.path.abspath(os.path.curdir) + os.sep
+            if os.access(path, os.R_OK):
+                os.chdir(path)
+                self._curdir = os.path.abspath(os.path.curdir) + os.sep
+            else:
+                # Doesn't have permissions
+                ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
+                                  (ed_glob.SB_INFO,
+                                   _("Can't change directory to: %s") % path))
+                wx.Bell()
+                self.Clear()
         else:
+            # Invalid path
             self.Clear()
             wx.Bell()
 
@@ -416,13 +433,14 @@ class CommandExecuter(eclib.CommandEntryBase):
         # e fname: edit file
         cmd = cmd[1:].strip()
         frame = self.GetTopLevelParent()
+        cmd = ebmlib.GetPathFromURI(cmd)
         if not os.path.isabs(cmd):
             cmd = os.path.join(self._curdir, cmd)
 
-        if os.path.exists(cmd):
+        if ebmlib.PathExists(cmd):
             frame.DoOpen(ed_glob.ID_COMMAND_LINE_OPEN, cmd)
         else:
-            frame.nb.OpenPage(util.GetPathName(cmd), util.GetFileName(cmd))
+            frame.nb.OpenPage(ebmlib.GetPathName(cmd), ebmlib.GetFileName(cmd))
 
     def ExecuteCommand(self, cmd_str):
         """Interprets and executes a command then hides the control
@@ -443,6 +461,9 @@ class CommandExecuter(eclib.CommandEntryBase):
                 self.Quit()
         elif cmd.startswith(u'e '):
             self.EditCommand(cmd)
+        elif cmd.rstrip() == u'e!':
+            ctrl = frame.nb.GetCurrentCtrl()
+            ctrl.RevertToSaved()
         elif self.RE_GO_WIN.match(cmd):
             self.GoWindow(cmd)
         elif re.match(self.RE_GO_BUFFER, cmd):
@@ -574,6 +595,15 @@ class CommandExecuter(eclib.CommandEntryBase):
         if not os.path.isdir(head):
             return []
 
+        # Return empty list of user does not have
+        # read access to the directory
+        if not os.access(head, os.R_OK):
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
+                              (ed_glob.SB_INFO,
+                               _("Access Denied: %s") % head))
+            wx.Bell() # Beep to alert
+            return list()
+
         # We expanded head, so trim the suggestion list of its head
         # so we can add the tail of the suggestion back to the original head
         candidates = [os.path.basename(p) for p in os.listdir(head)
@@ -667,12 +697,25 @@ class CommandExecuter(eclib.CommandEntryBase):
         @param evt: event that called this handler
 
         """
+        e_key = evt.GetKeyCode()
+        if e_key == wx.WXK_ESCAPE:
+            evt.Skip()
+            return
+
         val = self.GetValue()
+        cwd_info = ""
+        if val.strip() in ['cwd', 'e', 'cd']:
+            cwd_info = "   " + _(u"cwd: ") + self._curdir
+        self.Parent.info_lbl.SetLabel(cwd_info)
         if self._popup.IsShown():
             if not len(val):
                 self._popup.Hide()
             else:
                 wx.CallAfter(self.UpdateAutoComp)
+        else:
+            if self._popup.HasSuggestions():
+                    self._AdjustValue(self._popup.GetSelection())
+            self.ListDir()
         self._AdjustSize()
         evt.Skip()
 
@@ -795,7 +838,64 @@ class LineCtrl(eclib.CommandEntryBase):
 #-----------------------------------------------------------------------------#
 # TODO: merge the common parts of these two classes into a single base class
 
-class PopupList(wx.MiniFrame):
+class PopupListBase(object):
+    """Common functionality between Popuplist GTK and Mac"""
+
+    def AdvanceSelection(self, next=True):
+        """Advance the list selection
+        @keyword next: goto the next or previous selection
+        @type next: bool
+
+        """
+        sel = self._list.GetSelection()
+        if next:
+            count = self._list.GetCount()
+            sel += 1
+            if sel < count:
+                self._list.SetSelection(sel)
+        else:
+            sel -= 1
+            if sel >= 0:
+                self._list.SetSelection(sel)
+
+    def GetSelection(self):
+        """Get the string that is currently selected in the list
+        @return: string selection
+
+        """
+        return self._list.GetStringSelection()
+
+    def HasSelection(self):
+        """Tells whether anything in the list is selected"""
+        return self._list.GetSelection() != wx.NOT_FOUND
+
+    def HasSuggestions(self):
+        """Tell whether the list is showing suggestions"""
+        return self.IsShown() and self.ListCount() > 0
+
+    def ListCount(self):
+        """return the number of elements in the popup list"""
+        return self._list.GetCount()
+
+    def GetListCtrl(self):
+        """Get the ListBox control of the popupwindow"""
+        return self._list
+
+    def GetChoices(self):
+        """Get the items as a list
+        @return: list of strings
+
+        """
+        return self._list.GetStrings()
+
+    def SetSelection(self, index):
+        """Set the selection in the list by index
+        @param index: zero based index to set selection by
+
+        """
+        self._list.SetSelection(index)
+
+class PopupList(wx.MiniFrame, PopupListBase):
     """Popup window with a listbox in it"""
     def __init__(self, parent, choices=list(), pos=wx.DefaultPosition):
 
@@ -806,6 +906,7 @@ class PopupList(wx.MiniFrame):
             style = style | wx.SIMPLE_BORDER
 
         wx.MiniFrame.__init__(self, parent, pos=pos, style=style)
+        PopupListBase.__init__(self)
 
         # Attributes
         self._list = wx.ListBox(self, choices=choices,
@@ -852,52 +953,6 @@ class PopupList(wx.MiniFrame):
         parent.Raise()
         parent.SetFocus()
 
-    def AdvanceSelection(self, down=True):
-        """Advance the selection in the list
-        @keyword down: move selection down or up
-
-        """
-        csel = self._list.GetSelection()
-        if csel != wx.NOT_FOUND:
-            if down:
-                csel += 1
-            else:
-                csel -= 1
-                csel = max(csel, 0)
-
-            if csel < len(self._list.GetItems()):
-                self._list.SetSelection(csel)
-
-    def GetChoices(self):
-        """Get the items as a list
-        @return: list of strings
-
-        """
-        return self._list.GetStrings()
-
-    def GetListCtrl(self):
-        """Get the ListBox control of the popupwindow"""
-        return self._list
-
-    def GetSelection(self):
-        """Get the string that is currently selected in the list
-        @return: string selection
-
-        """
-        return self._list.GetStringSelection()
-
-    def HasSelection(self):
-        """Tells whether anything in the list is selected"""
-        return self._list.GetSelection() != wx.NOT_FOUND
-
-    def HasSuggestions(self):
-        """Tell whether the list is showing suggestions"""
-        return self.IsShown() and self.ListCount() > 0
-
-    def ListCount(self):
-        """return the number of elements in the popup list"""
-        return self._list.GetCount()
-
     def OnFocus(self, evt):
         """Raise and reset the focus to the parent window whenever
         we get focus.
@@ -961,13 +1016,6 @@ class PopupList(wx.MiniFrame):
         if count > 0:
             self._list.SetSelection(selection)
 
-    def SetSelection(self, index):
-        """Set the selection in the list by index
-        @param index: zero based index to set selection by
-
-        """
-        self._list.SetSelection(index)
-
     def SetStringSelection(self, text):
         """Set the list selection by using a string value
         @param text: string to select in list
@@ -990,11 +1038,12 @@ class PopupList(wx.MiniFrame):
 
 #----------------------------------------------------------------------------#
 
-class PopupWinList(wx.PopupWindow):
+class PopupWinList(wx.PopupWindow, PopupListBase):
     """Popuplist for Windows/GTK"""
     def __init__(self, parent, choices=list(), pos=wx.DefaultPosition):
         """Create the popup window and its list control"""
         wx.PopupWindow.__init__(self, parent)
+        PopupListBase.__init__(self)
 
         # Attributes
         self._list = wx.ListBox(self, choices=choices, pos=(0, 0),
@@ -1012,58 +1061,6 @@ class PopupWinList(wx.PopupWindow):
         # Event Handlers
         self.Bind(wx.EVT_SIZE, self.OnSize)
 
-    def AdvanceSelection(self, down=True):
-        """Advance the selection in the list
-        @keyword down: move selection down or up
-
-        """
-        item_count = self._list.GetCount()
-        if item_count == 0:
-            return
-        csel = self._list.GetSelection()
-        if csel == wx.NOT_FOUND:
-            if down:
-                csel = 0
-            else:
-                csel = -1
-        else:
-            if down:
-                csel += 1
-            else:
-                csel -= 1
-
-        # If it's -1 actually, but just in case something
-        # crazy happens and it drops even below -1
-        if csel < 0:
-            csel = item_count - 1
-        if csel >= item_count:
-            csel = 0
-        self._list.SetSelection(csel)
-        self._list.EnsureVisible(csel)
-
-    def GetListCtrl(self):
-        """Get the ListBox control of the popupwindow"""
-        return self._list
-
-    def GetSelection(self):
-        """Get the string that is currently selected in the list
-        @return: string selection
-
-        """
-        return self._list.GetStringSelection()
-
-    def HasSelection(self):
-        """Tells whether anything in the list is selected"""
-        return self._list.GetSelection() != wx.NOT_FOUND
-
-    def HasSuggestions(self):
-        """Tell whether the list is showing suggestions"""
-        return self.IsShown() and self.ListCount() > 0
-
-    def ListCount(self):
-        """return the number of elements in the popup list"""
-        return self._list.GetCount()
-
     def OnSize(self, evt):
         """Resize the list box to the correct size to fit."""
         csz = self.GetClientSize()
