@@ -14,8 +14,8 @@ Text editor buffer view control for the main notebook
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_editv.py 59800 2009-03-24 05:47:46Z CJP $"
-__revision__ = "$Revision: 59800 $"
+__svnid__ = "$Id: ed_editv.py 61041 2009-06-13 08:20:03Z CJP $"
+__revision__ = "$Revision: 61041 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -29,12 +29,28 @@ import ed_stc
 import ed_tab
 from doctools import DocPositionMgr
 from profiler import Profile_Get
-from util import GetFileModTime, Log, SetClipboardText
+from util import Log, SetClipboardText
+from ebmlib import GetFileModTime
 
 #--------------------------------------------------------------------------#
 
 _ = wx.GetTranslation
 
+def modalcheck(func):
+    """Decorator method to add extra modality guards to functions that
+    show modal dialogs. Arg 0 must be a Window instance.
+
+    """
+    def WrapModal(*args, **kwargs):
+        self = args[0]
+        self._has_dlg = True
+        func(*args, **kwargs)
+        self._has_dlg = False
+
+    WrapModal.__name__ = func.__name__
+    WrapModal.__doc__ = func.__doc__
+    return WrapModal
+
 #--------------------------------------------------------------------------#
 
 class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
@@ -46,10 +62,11 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
                  size=wx.DefaultSize, style=0, use_dt=True):
         """Initialize the editor view"""
         ed_stc.EditraStc.__init__(self, parent, id_, pos, size, style, use_dt)
-        ed_tab.EdTabBase.__init__(self, parent)
+        ed_tab.EdTabBase.__init__(self)
 
         # Attributes
         self._ignore_del = False
+        self._has_dlg = False
 
         # Initialize the classes position manager for the first control
         # that is created only.
@@ -81,11 +98,16 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
 
     def DoOnIdle(self):
         """Check if the file has been modified and prompt a warning"""
-        if Profile_Get('CHECKMOD'):
+        # Don't check while the file is loading
+        if self.IsLoading():
+            return
+
+        if not self._has_dlg and Profile_Get('CHECKMOD'):
             cfile = self.GetFileName()
             lmod = GetFileModTime(cfile)
             mtime = self.GetModTime()
             if mtime and not lmod and not os.path.exists(cfile):
+                # File was deleted since last check
                 wx.CallAfter(self.PromptToReSave, cfile)
             elif mtime < lmod:
                 # Check if we should automatically reload the file or not
@@ -94,9 +116,24 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
                     wx.CallAfter(self.DoReloadFile)
                 else:
                     wx.CallAfter(self.AskToReload, cfile)
-            else:
-                pass
 
+        # Check for changes to permissions
+        readonly = self._nb.ImageIsReadOnly(self.GetTabIndex())
+        if self.File.IsReadOnly() != readonly:
+            if readonly:
+                # File is no longer read only
+                self._nb.SetPageImage(self.GetTabIndex(),
+                                      str(self.GetLangId()))
+            else:
+                # File has changed to be readonly
+                self._nb.SetPageImage(self.GetTabIndex(),
+                                      ed_glob.ID_READONLY)
+            self._nb.Refresh()
+
+        else:
+            pass
+
+    @modalcheck
     def DoReloadFile(self):
         """Reload the current file"""
         ret, rmsg = self.ReloadFile()
@@ -163,7 +200,7 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         if not len(title):
             title = fname = self.GetTabLabel()
 
-        if self.GetModify():
+        if self.GetModify() and not title.startswith(u'*'):
             title = u"*" + title
         return u"%s - file://%s" % (title, fname)
 
@@ -216,6 +253,7 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         self.PopupMenu(EdEditorView.RCLICK_MENU)
         evt.Skip()
 
+    @modalcheck
     def PromptToReSave(self, cfile):
         """Show a dialog prompting to resave the current file
         @param cfile: the file in question
@@ -235,9 +273,9 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         else:
             self.SetModTime(0)
 
+    @modalcheck
     def AskToReload(self, cfile):
         """Show a dialog asking if the file should be reloaded
-        @param win: Window to prompt dialog on top of
         @param cfile: the file to prompt for a reload of
 
         """
