@@ -15,8 +15,8 @@ main Ui component of the editor that contains all the other components.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 60367 2009-04-26 00:20:46Z CJP $"
-__revision__ = "$Revision: 60367 $"
+__svnid__ = "$Id: ed_main.py 61179 2009-06-23 22:47:20Z CJP $"
+__revision__ = "$Revision: 61179 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -44,6 +44,7 @@ import generator
 import plugin
 import perspective as viewmgr
 import iface
+import ebmlib
 import eclib
 
 # Function Aliases
@@ -58,6 +59,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
     @todo: modularize the event handling more (pubsub?)
 
     """
+    # Clipboard ring is limited to 25, why? Because any more is a waste of
+    # memory and an inefficient waste of your time to cyle through.
+    CLIPBOARD = util.EdClipboard(25)
     PRINTER = None
 
     def __init__(self, parent, id_, wsize, title):
@@ -173,6 +177,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
                                        # Edit Menu
                                        (ID_PASTE_AFTER, self.DispatchToControl),
+                                       (ID_CYCLE_CLIPBOARD, self.DispatchToControl),
                                        (ID_COLUMN_MODE, self.DispatchToControl),
                                        (ID_TOGGLE_FOLD, self.DispatchToControl),
                                        (ID_TOGGLE_ALL_FOLDS, self.DispatchToControl),
@@ -220,6 +225,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                      (ID_CUT, self.OnUpdateClipboardUI),
                                      (ID_PASTE, self.OnUpdateClipboardUI),
                                      (ID_PASTE_AFTER, self.OnUpdateClipboardUI),
+                                     (ID_CYCLE_CLIPBOARD, self.OnUpdateClipboardUI),
                                      (ID_UNDO, self.OnUpdateClipboardUI),
                                      (ID_REDO, self.OnUpdateClipboardUI),
                                      (ID_COLUMN_MODE, self.OnUpdateClipboardUI),
@@ -442,15 +448,15 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                     if _PGET('OPEN_NW', default=False):
                         wx.GetApp().OpenNewWindow(path)
                     else:
-                        self.nb.OpenPage(util.GetPathName(path),
-                                         util.GetFileName(path))
+                        self.nb.OpenPage(ebmlib.GetPathName(path),
+                                         ebmlib.GetFileName(path))
                         self.nb.GoCurrentPage()
 
             dlg.Destroy()
         else:
             self.LOG("[ed_main][info] CMD Open File: %s" % fname)
-            self.nb.OpenPage(util.GetPathName(fname),
-                             util.GetFileName(fname), quiet=True)
+            self.nb.OpenPage(ebmlib.GetPathName(fname),
+                             ebmlib.GetFileName(fname), quiet=True)
             self.Raise()
 
     def GetCommandbar(self):
@@ -520,7 +526,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 if isinstance(fname, basestring) and fname:
                     # TODO: find out why these errors are happening
                     #       when loading the pickled strings on some systems
-                    #       The pickled strings are in unicode format.
+                    #       The pickled strings are not in unicode format.
                     try:
                         self.filehistory.AddFileToHistory(fname)
                     except wx.PyAssertionError:
@@ -616,7 +622,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             return
 
         for ctrl in ctrls:
-            fname = util.GetFileName(ctrl[1].GetFileName())
+            fname = ebmlib.GetFileName(ctrl[1].GetFileName())
             if fname != '':
                 fpath = ctrl[1].GetFileName()
                 result = ctrl[1].SaveFile(fpath)
@@ -663,7 +669,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             dlg.Destroy()
 
             result = ctrl.SaveFile(path)
-            fname = util.GetFileName(ctrl.GetFileName())
+            fname = ebmlib.GetFileName(ctrl.GetFileName())
             if not result:
                 err = ctrl.GetDocument().GetLastError()
                 ed_mdlg.SaveErrorDlg(self, fname, err)
@@ -693,7 +699,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                 wx.SAVE | wx.OVERWRITE_PROMPT)
 
             if dlg.ShowModal() == wx.ID_OK:
-                profiler.Profile().Write(dlg.GetPath())
+                profiler.TheProfile.Write(dlg.GetPath())
                 self.PushStatusText(_("Profile Saved as: %s") % \
                                     dlg.GetFilename(), SB_INFO)
             dlg.Destroy()
@@ -714,7 +720,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
             result = dlg.ShowModal()
             if result == wx.ID_OK:
-                profiler.Profile().Load(dlg.GetPath())
+                profiler.TheProfile.Load(dlg.GetPath())
                 self.PushStatusText(_("Loaded Profile: %s") % \
                                     dlg.GetFilename(), SB_INFO)
             dlg.Destroy()
@@ -900,7 +906,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         # Update profile
         ppath = _PGET('MYPROFILE')
-        profiler.Profile().Write(ppath)
+        profiler.TheProfile.Write(ppath)
         self.LOG("[ed_main][info] Saving profile to %s" % ppath)
 
         # Post exit notice to all aui panes
@@ -966,6 +972,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         paneInfo = self._mgr.GetPane("EditPane")
         if paneInfo.IsMaximized():
             self._mgr.RestorePane(paneInfo)
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_RESTORE, context=self.GetId())
         else:
             self._mgr.RestoreMaximizedPane()
             self._mgr.MaximizePane(paneInfo)
@@ -1077,27 +1084,50 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                         ID_SELECTALL, ID_UNDO, ID_REDO, ID_CUT, ID_COPY,
                         ID_PASTE, ID_LINE_BEFORE, ID_LINE_AFTER, ID_DUP_LINE,
                         ID_PASTE_AFTER, ID_COLUMN_MODE, ID_TOGGLE_FOLD,
+                        ID_CYCLE_CLIPBOARD,
                         ID_TOGGLE_ALL_FOLDS, ID_DELETE_LINE ]
 
         # Special handling for common clipboard related actions
         has_focus = self.FindFocus()
-        if has_focus != ctrl and e_id in active_only:
-            if has_focus is not None:
-                if e_id == ID_PASTE and hasattr(has_focus, 'Paste'):
-                    has_focus.Paste()
-                elif e_id == ID_CUT and hasattr(has_focus, 'Cut'):
-                    has_focus.Cut()
-                elif e_id == ID_COPY and hasattr(has_focus, 'Copy'):
-                    has_focus.Copy()
-                elif e_id == ID_REDO and hasattr(has_focus, 'Redo'):
-                    has_focus.Redo()
-                elif e_id == ID_UNDO and hasattr(has_focus, 'Undo'):
-                    has_focus.Undo()
+        if has_focus is not None:
+            if e_id == ID_PASTE and hasattr(has_focus, 'Paste'):
+                has_focus.Paste()
+                return
+            elif e_id == ID_CYCLE_CLIPBOARD:
+                start, end = has_focus.GetSelection()
+                start, end = min(start, end), max(start, end)
+                txt = has_focus.GetRange(start, end)
+                if not MainWindow.CLIPBOARD.IsAtIndex(txt):
+                    MainWindow.CLIPBOARD.Reset()
+
+                next = MainWindow.CLIPBOARD.GetNext()
+                if isinstance(has_focus, wx.stc.StyledTextCtrl):
+                    has_focus.ReplaceSelection(next)
+                elif hasattr(has_focus, 'Replace'):
+                    has_focus.Replace(start, end, next)
                 else:
-                    evt.Skip()
-            else:
-                evt.Skip()
-            return
+                    return
+
+                has_focus.SetSelection(start, start+len(next))
+                return
+            elif e_id == ID_CUT and hasattr(has_focus, 'Cut'):
+                start, end = has_focus.GetSelection()
+                txt = has_focus.GetRange(start, end)
+                MainWindow.CLIPBOARD.Put(txt)
+                has_focus.Cut()
+                return
+            elif e_id == ID_COPY and hasattr(has_focus, 'Copy'):
+                start, end = has_focus.GetSelection()
+                txt = has_focus.GetRange(start, end)
+                MainWindow.CLIPBOARD.Put(txt)
+                has_focus.Copy()
+                return
+            elif e_id == ID_REDO and hasattr(has_focus, 'Redo'):
+                has_focus.Redo()
+                return
+            elif e_id == ID_UNDO and hasattr(has_focus, 'Undo'):
+                has_focus.Undo()
+                return
 
         menu_ids = list(syntax.SYNTAX_IDS)
         menu_ids.extend([ID_SHOW_EOL, ID_SHOW_WS, ID_INDENT_GUIDES, ID_SYNTAX,
@@ -1108,7 +1138,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                          ID_TRIM_WS, ID_SHOW_EDGE, ID_MACRO_START,
                          ID_MACRO_STOP, ID_MACRO_PLAY, ID_TO_LOWER,
                          ID_TO_UPPER, ID_USE_SOFTTABS,
-                         ID_GOTO_MBRACE, ID_HLCARET_LINE, ID_REVERT_FILE
+                         ID_GOTO_MBRACE, ID_HLCARET_LINE, ID_REVERT_FILE,
                          ])
         menu_ids.extend(active_only)
 
@@ -1215,7 +1245,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             if hasattr(focus, 'CanRedo'):
                 enable = focus.CanRedo()
             evt.Enable(enable)
-        elif e_id in (ID_PASTE, ID_PASTE_AFTER):
+        elif e_id in (ID_PASTE, ID_PASTE_AFTER, ID_CYCLE_CLIPBOARD):
             if hasattr(focus, 'CanPaste'):
                 enable = focus.CanPaste()
             evt.Enable(enable)
@@ -1420,6 +1450,21 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             name = " - " + name
         wx.Frame.SetTitle(self, title + name)
 
+    @classmethod
+    def UpdateClipboardRing(cls):
+        """Update the clipboard ring to sync it with the
+        system clipboard.
+        @note: for internal use only
+
+        """
+        txt = util.GetClipboardText()
+        if txt is None or cls.CLIPBOARD.IsAtIndex(txt):
+            return
+
+        # Something new has come in from an external program
+        cls.CLIPBOARD.Reset()
+        cls.CLIPBOARD.Put(txt)
+
 #-----------------------------------------------------------------------------#
 # Event handlers that don't need to be part of the class
 
