@@ -13,8 +13,8 @@ This class implements Editra's main notebook control.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 60397 2009-04-26 19:20:56Z CJP $"
-__revision__ = "$Revision: 60397 $"
+__svnid__ = "$Id: ed_pages.py 61142 2009-06-20 05:28:36Z CJP $"
+__revision__ = "$Revision: 61142 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -34,15 +34,19 @@ import util
 import ed_msg
 import ed_txt
 import ed_mdlg
+import ebmlib
 import eclib
 from extern import flatnotebook as FNB
 
 #--------------------------------------------------------------------------#
 # Globals
+ID_IDLE_TIMER = wx.NewId()
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
+#--------------------------------------------------------------------------#
+
 class EdPages(FNB.FlatNotebook):
     """Editras editor buffer botebook
     @todo: allow for tab styles to be configurable (maybe)
@@ -64,6 +68,7 @@ class EdPages(FNB.FlatNotebook):
 
         # Notebook attributes
         self.LOG = wx.GetApp().GetLog()
+        self._idletimer = wx.Timer(self, ID_IDLE_TIMER)
         self.DocMgr = ed_editv.EdEditorView.DOCMGR
         self._searchctrl = ed_search.SearchController(self, self.GetCurrentCtrl)
         self._searchctrl.SetLookinChoices(Profile_Get('SEARCH_LOC',
@@ -101,20 +106,23 @@ class EdPages(FNB.FlatNotebook):
         self._pages.Bind(wx.EVT_MIDDLE_UP, self.OnMClick)
         self.Bind(FNB.EVT_FLATNOTEBOOK_PAGE_CONTEXT_MENU, self.OnTabMenu)
         self.Bind(wx.EVT_MENU, self.OnMenu)
-        self.Bind(wx.EVT_IDLE, self.OnIdle)
+        self.Bind(wx.EVT_TIMER, self.OnIdle, id=ID_IDLE_TIMER)
 
         # Message handlers
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_NOTEBOOK)
+        ed_msg.Subscribe(self.OnUpdatePosCache, ed_msg.EDMSG_UI_STC_POS_JUMPED)
         ed_msg.RegisterCallback(self.OnDocPointerRequest, ed_msg.EDREQ_DOCPOINTER)
 
         # Add a blank page
         self.NewPage()
+        self._idletimer.Start(333) # Do idle checks 3 times a second
 
     #---- End Init ----#
 
     def __del__(self):
         ed_msg.Unsubscribe(self.OnThemeChanged)
+        ed_msg.Unsubscribe(self.OnUpdatePosCache)
         ed_msg.UnRegisterCallback(self.OnDocPointerRequest)
 
     #---- Function Definitions ----#
@@ -268,7 +276,10 @@ class EdPages(FNB.FlatNotebook):
                  (ed_glob.ID_FIND_REPLACE, self._searchctrl.OnShowFindDlg),
                  (ed_glob.ID_FIND_NEXT, self._searchctrl.OnFind),
                  (ed_glob.ID_FIND_PREVIOUS, self._searchctrl.OnFind),
-                 (ed_glob.ID_FIND_SELECTED, self._searchctrl.OnFindSelected)]
+                 (ed_glob.ID_FIND_SELECTED, self._searchctrl.OnFindSelected),
+                 (ed_glob.ID_NEXT_POS, self.OnNavigateToPos),
+                 (ed_glob.ID_PRE_POS, self.OnNavigateToPos)]
+
         return rlist
 
     def GetUiHandlers(self):
@@ -277,7 +288,9 @@ class EdPages(FNB.FlatNotebook):
 
         """
         return [(ed_glob.ID_FIND_NEXT, self._searchctrl.OnUpdateFindUI),
-                (ed_glob.ID_FIND_PREVIOUS, self._searchctrl.OnUpdateFindUI)]
+                (ed_glob.ID_FIND_PREVIOUS, self._searchctrl.OnUpdateFindUI),
+                (ed_glob.ID_NEXT_POS, self.OnUpdateNaviUI),
+                (ed_glob.ID_PRE_POS, self.OnUpdateNaviUI)]
 
     def InsertPage(self, index, page, text, select=True, imageId=-1):
         """Insert a page into the notebook"""
@@ -293,7 +306,7 @@ class EdPages(FNB.FlatNotebook):
         try:
             f_handle = open(session, 'wb')
         except (IOError, OSError), msg:
-            return (_("Error Loading Session File"),  unicode(msg))
+            return (_("Error Saving Session File"),  ed_txt.DecodeString(msg))
 
         try:
             sdata = dict(win1=self.GetFileNames())
@@ -446,6 +459,46 @@ class EdPages(FNB.FlatNotebook):
         else:
             evt.Skip()
 
+    def OnNavigateToPos(self, evt):
+        """Handle buffer position history navigation events"""
+        e_id = evt.GetId()
+        fname, pos = (None, None)
+        cname = self.control.GetFileName()
+        cpos = self.control.GetCurrentPos()
+        if e_id == ed_glob.ID_NEXT_POS:
+            if self.DocMgr.CanNavigateNext():
+                fname, pos = self.DocMgr.GetNextNaviPos()
+                if (fname, pos) == (cname, cpos):
+                    fname, pos = (None, None)
+                    tmp = self.DocMgr.GetNextNaviPos()
+                    if tmp is not None:
+                        fname, pos = tmp
+        elif e_id == ed_glob.ID_PRE_POS:
+            if self.DocMgr.CanNavigatePrev():
+                fname, pos = self.DocMgr.GetPreviousNaviPos()
+                if (fname, pos) == (cname, cpos):
+                    fname, pos = (None, None)
+                    tmp = self.DocMgr.GetPreviousNaviPos()
+                    if tmp is not None:
+                        fname, pos = tmp
+        else:
+            evt.Skip()
+            return
+
+        ctrl = self.FindBuffer(fname)
+        if ctrl is None:
+            # Open the file in the editor
+            if fname is not None:
+                self.OpenPage(ebmlib.GetPathName(fname),
+                              ebmlib.GetFileName(fname))
+                self.control.SetCaretPos(pos)
+        else:
+            # Raise page to top and goto position
+            pages = [self.GetPage(page) for page in xrange(self.GetPageCount())]
+            idx = pages.index(ctrl)
+            self.ChangePage(idx)
+            ctrl.SetCaretPos(pos)
+
     def OnTabMenu(self, evt):
         """Show the tab context menu"""
         # Destroy any existing menu
@@ -468,6 +521,30 @@ class EdPages(FNB.FlatNotebook):
         """
         self.UpdateAllImages()
 
+    def OnUpdatePosCache(self, msg):
+        """Update the position cache for buffer position changes
+        @param msg: message data
+
+        """
+        if self._ses_load:
+            return
+
+        tlw = self.GetTopLevelParent()
+        if tlw.GetId() == msg.GetContext():
+            data = msg.GetData()
+            self.DocMgr.AddNaviPosition(data['fname'], data['prepos'])
+            self.DocMgr.AddNaviPosition(data['fname'], data['pos'])
+
+    def OnUpdateNaviUI(self, evt):
+        """UpdateUI handler for position navigator"""
+        e_id = evt.GetId()
+        if e_id == ed_glob.ID_NEXT_POS:
+            evt.Enable(self.DocMgr.CanNavigateNext())
+        elif e_id == ed_glob.ID_PRE_POS:
+            evt.Enable(self.DocMgr.CanNavigatePrev())
+        else:
+            evt.Skip()
+
     def OpenDocPointer(self, ptr, doc, title=u''):
         """Open a page using an stc document poiner
         @param ptr: EdEditorView document Pointer
@@ -490,13 +567,13 @@ class EdPages(FNB.FlatNotebook):
         path = nbuff.GetFileName()
         if Profile_Get('SAVE_POS'):
             pos = self.DocMgr.GetPos(path)
-            nbuff.GotoPos(pos)
+            nbuff.SetCaretPos(pos)
             nbuff.ScrollToColumn(0)
 
         if title:
             filename = title
         else:
-            filename = util.GetFileName(path)
+            filename = ebmlib.GetFileName(path)
 
         if need_add:
             self.AddPage(nbuff, filename)
@@ -514,7 +591,9 @@ class EdPages(FNB.FlatNotebook):
         self.control = nbuff
         self.GoCurrentPage()
         self.GetTopLevelParent().Thaw()
-        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, nbuff.GetFileName())
+        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED,
+                           nbuff.GetFileName(),
+                           context=self.frame.GetId())
 
     def OpenFileObject(self, fileobj):
         """Open a new text editor page with the given file object. The file
@@ -529,7 +608,7 @@ class EdPages(FNB.FlatNotebook):
 
         # Load the files data
         path = fileobj.GetPath()
-        filename = util.GetFileName(path)
+        filename = ebmlib.GetFileName(path)
         control.SetDocument(fileobj)
         result = control.ReloadFile()
 
@@ -560,7 +639,9 @@ class EdPages(FNB.FlatNotebook):
 
         # Refocus on selected page
         self.GoCurrentPage()
-        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.control.GetFileName())
+        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED,
+                           self.control.GetFileName(),
+                           context=self.frame.GetId())
 
         if Profile_Get('WARN_EOL', default=True):
             self.control.CheckEOL()
@@ -575,6 +656,12 @@ class EdPages(FNB.FlatNotebook):
         """
         path2file = os.path.join(path, filename)
 
+        # Resolve links to real file
+        if ebmlib.IsLink(path2file):
+            path2file = ebmlib.ResolveRealPath(path2file)
+            path = ebmlib.GetPathName(path2file)
+            filename = ebmlib.GetFileName(path2file)
+
         if self.DocDuplicated(path2file):
             return
 
@@ -692,10 +779,12 @@ class EdPages(FNB.FlatNotebook):
         """Perform post file open actions"""
         if Profile_Get('SAVE_POS'):
             pos = self.DocMgr.GetPos(self.control.GetFileName())
-            self.control.GotoPos(pos)
+            self.control.SetCaretPos(pos)
             self.control.ScrollToColumn(0)
 
-        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.control.GetFileName())
+        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED,
+                           self.control.GetFileName(),
+                           context=self.frame.GetId())
 
     def GoCurrentPage(self):
         """Move Focus to Currently Selected Page.
@@ -792,6 +881,18 @@ class EdPages(FNB.FlatNotebook):
                 return True
         return False
 
+    def FindBuffer(self, fpath):
+        """Find the buffer containing the given file
+        @param fpath: full path of file to look for
+        @return: EdStc or None
+        @todo: handle matching based on the buffer control itself
+
+        """
+        for ctrl in self.GetTextControls():
+            if fpath == ctrl.GetFileName():
+                return ctrl
+        return None
+
     #---- Event Handlers ----#
     def OnDrop(self, files):
         """Opens dropped files
@@ -843,8 +944,8 @@ class EdPages(FNB.FlatNotebook):
                 valid_files.append(fname)
 
         for fname in valid_files:
-            pathname = util.GetPathName(fname)
-            the_file = util.GetFileName(fname)
+            pathname = ebmlib.GetPathName(fname)
+            the_file = ebmlib.GetFileName(fname)
             self.OpenPage(pathname, the_file)
             self.frame.PushStatusText(_("Opened file: %s") % fname, \
                                       ed_glob.SB_INFO)
@@ -861,15 +962,6 @@ class EdPages(FNB.FlatNotebook):
             if page is not None:
                 page.DoOnIdle()
 
-#            # Update document indicator state
-#            if hasattr(page, 'GetDocument'):
-#                doc = page.GetDocument()
-#                sel = self.GetSelection()
-#                img = self.GetPageImage(sel)
-#                roidx = self._index[ed_glob.ID_READONLY]
-#                if doc.ReadOnly and img != roidx:
-#                    FNB.FlatNotebook.SetPageImage(sel, roidx)
-
     def OnLeftUp(self, evt):
         """Traps clicks sent to page close buttons and
         redirects the action to the ClosePage function
@@ -908,7 +1000,9 @@ class EdPages(FNB.FlatNotebook):
         if isinstance(self.control, wx.Window):
             self.control.DoDeactivateTab()
 
-        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGING, (self,) + pages)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGING,
+                           (self,) + pages,
+                           context=self.frame.GetId())
 
     def ChangePage(self, pg_num, old=-2):
         """Change the page and focus to the the given page id
@@ -935,7 +1029,9 @@ class EdPages(FNB.FlatNotebook):
             cpage = old
 
         if not self.frame.IsExiting() and cpage != pg_num:
-            ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED, (self, pg_num))
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED,
+                               (self, pg_num),
+                               context=self.frame.GetId())
 
     def OnPageChanged(self, evt):
         """Actions to do after a page change
@@ -969,7 +1065,8 @@ class EdPages(FNB.FlatNotebook):
 
         evt.Skip()
         ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSING,
-                           (self, self.GetSelection()))
+                           (self, self.GetSelection()),
+                           context=self.frame.GetId())
 
     def OnPageClosed(self, evt):
         """Handles Paged Closed Event
@@ -981,7 +1078,9 @@ class EdPages(FNB.FlatNotebook):
         evt.Skip()
         self.LOG("[ed_pages][evt] Closed Page: #%d" % cpage)
         self.UpdateIndexes()
-        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSED, (self, cpage))
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSED,
+                           (self, cpage),
+                           context=self.frame.GetId())
 
     #---- End Event Handlers ----#
 
