@@ -15,8 +15,8 @@ text documents and files.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 60329 2009-04-25 01:53:20Z CJP $"
-__revision__ = "$Revision: 60329 $"
+__svnid__ = "$Id: ed_search.py 60840 2009-05-31 16:00:50Z CJP $"
+__revision__ = "$Revision: 60840 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -42,6 +42,51 @@ import ebmlib
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
+class EdSearchEngine(ebmlib.SearchEngine):
+    def __init__(self, query, regex=True, down=True,
+                 matchcase=True, wholeword=False):
+        ebmlib.SearchEngine.__init__(self, query, regex, down,
+                                     matchcase, wholeword)
+        # Atttributes
+        self._offset = 0
+
+    def FormatResult(self, fname, lnum, match):
+        """Format the search result string for find all action that is performed
+        on a selection.
+        @return: string
+        @todo: better unicode handling
+
+        """
+        fname = ed_txt.DecodeString(fname, sys.getfilesystemencoding())
+        if not isinstance(fname, types.UnicodeType):
+            fname = _("DECODING ERROR")
+
+        match = ed_txt.DecodeString(match)
+        if not isinstance(match, types.UnicodeType):
+            match = _("DECODING ERROR")
+        else:
+            match = u" " + match.lstrip()
+
+        rstring = u"%(fname)s (%(lnum)d): %(match)s"
+        lnum = lnum + self._offset + 1
+        return rstring % dict(fname=fname, lnum=lnum, match=match)
+
+    def SetOffset(self, offset):
+        """Set the offset for a search in selection action
+        @param offset: int
+
+        """
+        self._offset = offset
+
+    def SetQuery(self, query):
+        """Set the query string"""
+        if isinstance(query, types.UnicodeType):
+            # Encode to UTF-8 as used internally by the stc
+            query = query.encode('utf-8')
+        super(EdSearchEngine, self).SetQuery(query)
+
+#--------------------------------------------------------------------------#
+
 class SearchController(object):
     """Controls the interface to the text search engine"""
     def __init__(self, owner, getstc):
@@ -62,7 +107,10 @@ class SearchController(object):
         self._li_sel   = 0
         self._filters  = None
         self._clients = list()
-        self._engine = None
+        self._engine = EdSearchEngine(u"") # For incremental searches
+
+        # Setup
+        self._engine.SetResultFormatter(self._engine.FormatResult)
 
         # Event handlers
         self._parent.Bind(eclib.EVT_FIND, self.OnFind)
@@ -227,9 +275,19 @@ class SearchController(object):
 
         # Create the search engine
         query = evt.GetFindString()
+        mode = evt.GetSearchType()
         engine = ebmlib.SearchEngine(query, evt.IsRegEx(),
                                      True, evt.IsMatchCase(), evt.IsWholeWord())
-        engine.SetSearchPool(stc.GetTextRaw())
+
+        if mode == eclib.LOCATION_CURRENT_DOC:
+            engine.SetSearchPool(stc.GetTextRaw())
+        elif mode == eclib.LOCATION_IN_SELECTION:
+            engine.SetSearchPool(stc.GetSelectedTextRaw())
+        else:
+            # TODO: report that this is not supported yet
+            #       this case should not happen as the count button is currently
+            #       disabled for any conditions that fall into this case.
+            return
 
         matches = engine.FindAll()
         if matches:
@@ -258,8 +316,7 @@ class SearchController(object):
             if not findnext and evt.GetId() == ed_glob.ID_FIND_PREVIOUS:
                 flags |= eclib.AFR_UP
 
-            evt = eclib.FindEvent(eclib.edEVT_FIND_NEXT,
-                                    flags=flags)
+            evt = eclib.FindEvent(eclib.edEVT_FIND_NEXT, flags=flags)
             evt.SetFindString(data.GetFindString())
 
         stc = self._stc()
@@ -267,22 +324,23 @@ class SearchController(object):
 
         # Create the search engine
         isdown = not evt.IsUp()
-        engine = ebmlib.SearchEngine(data.GetFindString(), evt.IsRegEx(),
-                                     isdown, evt.IsMatchCase(),
-                                     evt.IsWholeWord())
-        engine.SetResultFormatter(FormatResult)
-        self._engine = engine
+        self._engine.SetQuery(data.GetFindString())
+        self._engine.SetFlags(isregex=evt.IsRegEx(),
+                              matchcase=evt.IsMatchCase(),
+                              wholeword=evt.IsWholeWord(),
+                              down=isdown)
 
         # Check if expression was valid or not
-        if engine.GetQueryObject() is None:
-            wx.MessageBox(_("Invalid expression \"%s\"") % engine.GetQuery(),
+        if self._engine.GetQueryObject() is None:
+            fail = ed_txt.DecodeString(self._engine.GetQuery(), 'utf-8')
+            wx.MessageBox(_("Invalid expression \"%s\"") % fail,
                           _("Regex Compile Error"),
                           style=wx.OK|wx.CENTER|wx.ICON_ERROR)
             return
 
         # XXX: may be inefficent to copy whole buffer each time for files
         #      that are large.
-        engine.SetSearchPool(stc.GetTextRaw())
+        self._engine.SetSearchPool(stc.GetTextRaw())
 
         # Check if the direction changed
         ldir = self._posinfo['ldir']
@@ -304,7 +362,7 @@ class SearchController(object):
                     spos = end
 
         # Do the find
-        match = engine.Find(spos)
+        match = self._engine.Find(spos)
         if match is not None:
             start, end = match
 
@@ -327,11 +385,11 @@ class SearchController(object):
         else:
             # try search from top again
             if isdown:
-                match = engine.Find(0)
+                match = self._engine.Find(0)
                 ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
                                   (ed_glob.SB_INFO, _("Search wrapped to top")))
             else:
-                match = engine.Find(-1)
+                match = self._engine.Find(-1)
                 ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
                                   (ed_glob.SB_INFO,
                                   _("Search wrapped to bottom")))
@@ -352,9 +410,10 @@ class SearchController(object):
                 stc.EnsureVisible(line)
             else:
                 self._posinfo['found'] = -1
+                fail = ed_txt.DecodeString(self._engine.GetQuery(), 'utf-8')
                 ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
-                                  (ed_glob.SB_INFO,
-                                  _("\"%s\" was not found") % engine.GetQuery()))
+                                   (ed_glob.SB_INFO,
+                                   _("\"%s\" was not found") % fail))
 
     def OnFindAll(self, evt):
         """Find all results for the given query and display results in a
@@ -367,9 +426,9 @@ class SearchController(object):
             return
 
         # Create a new search engine object
-        engine = ebmlib.SearchEngine(query, evt.IsRegEx(), True,
-                                     evt.IsMatchCase(), evt.IsWholeWord())
-        engine.SetResultFormatter(FormatResult)
+        engine = EdSearchEngine(query, evt.IsRegEx(), True,
+                                evt.IsMatchCase(), evt.IsWholeWord())
+        engine.SetResultFormatter(engine.FormatResult)
 
         # Send the search function over to any interested parties that wish
         # to process the results.
@@ -383,6 +442,14 @@ class SearchController(object):
                 engine.SetSearchPool(stc.GetTextRaw())
                 ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH,
                                    (engine.FindAllLines,))
+        if smode == eclib.LOCATION_IN_SELECTION:
+            stc = self._stc()
+            sel_s = min(stc.GetSelection())
+            offset = stc.LineFromPosition(sel_s)
+            engine.SetOffset(offset)
+            engine.SetSearchPool(stc.GetSelectedTextRaw())
+            ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH,
+                               (engine.FindAllLines,))
         elif smode == eclib.LOCATION_OPEN_DOCS:
             files = [fname.GetFileName()
                      for fname in self._parent.GetTextControls()]
@@ -432,9 +499,9 @@ class SearchController(object):
             Profile_Set('SEARCH_FILTER', self._filters)
 
             # Destroy it
-            self._finddlg.Destroy()
+#            self._finddlg.Destroy()
 
-        self._finddlg = None
+#        self._finddlg = None
         evt.Skip()
 
     def OnOptionChanged(self, evt):
@@ -460,12 +527,16 @@ class SearchController(object):
                 value = replacestring
         else:
             value = replacestring
+
+        sel = self._stc().GetSelection()
+        if sel[0] == sel[1]:
+            return
+
         self._stc().ReplaceSelection(value)
 
         # Go to the next match
         # Fake event object for on Find Handler
-        tevt = eclib.FindEvent(eclib.edEVT_FIND_NEXT,
-                                 ed_glob.ID_FIND_PREVIOUS)
+        tevt = eclib.FindEvent(eclib.edEVT_FIND_NEXT, ed_glob.ID_FIND_PREVIOUS)
         self.OnFind(tevt, True)
 
     def OnReplaceAll(self, evt):
@@ -475,12 +546,11 @@ class SearchController(object):
         """
         smode = evt.GetSearchType()
         rstring = evt.GetReplaceString()
-        engine = ebmlib.SearchEngine(evt.GetFindString(), evt.IsRegEx(),
-                                     True, evt.IsMatchCase(), evt.IsWholeWord())
-        engine.SetResultFormatter(FormatResult)
+        engine = EdSearchEngine(evt.GetFindString(), evt.IsRegEx(),
+                                True, evt.IsMatchCase(), evt.IsWholeWord())
+        engine.SetResultFormatter(engine.FormatResult)
 
         results = 0
-
         if smode == eclib.LOCATION_CURRENT_DOC:
             stc = self._stc()
             engine.SetSearchPool(stc.GetTextRaw())
@@ -488,6 +558,26 @@ class SearchController(object):
             if matches is not None:
                 self.ReplaceInStc(stc, matches, rstring, evt.IsRegEx())
                 results = len(matches)
+        elif smode == eclib.LOCATION_IN_SELECTION:
+            stc = self._stc()
+            engine.SetSearchPool(stc.GetSelectedTextRaw())
+            matches = engine.FindAll()
+            if matches is not None:
+                self.ReplaceInStcSelection(stc, matches, rstring, evt.IsRegEx())
+                results = len(matches)
+#            regex = engine.GetQueryObject()
+#            if regex is not None:
+#                text = engine.GetSearchPool()
+#                def replaceString(match):
+#                    if evt.IsRegEx():
+#                        value = match.expand(rstring.encode('utf-8')).decode('utf-8')
+#                    else:
+#                        value = rstring
+#                    return value
+#                text = regex.sub(replaceString, text)
+#                stc.ReplaceSelection(text)
+            else:
+                pass # TODO: notify of no matches?
         elif smode == eclib.LOCATION_OPEN_DOCS:
             for ctrl in self._parent.GetTextControls():
                 engine.SetSearchPool(ctrl.GetTextRaw())
@@ -545,6 +635,7 @@ class SearchController(object):
         else:
             # Dialog is open already so just update it
             self._UpdateDialogState(eid)
+            self._finddlg.Show()
             self._finddlg.Raise()
         self._finddlg.SetFocus()
 
@@ -573,7 +664,7 @@ class SearchController(object):
     def ReplaceInStc(stc, matches, rstring, isregex=True):
         """Replace the strings at the position in the given StyledTextCtrl
         @param stc: StyledTextCtrl
-        @param matches: list of tuples [(s1, e1), (s2, e2)]
+        @param matches: list of match objects
         @param rstring: Replace string
 
         """
@@ -589,6 +680,24 @@ class SearchController(object):
             stc.ReplaceTarget(value)
         stc.EndUndoAction()
 
+    @staticmethod
+    def ReplaceInStcSelection(stc, matches, rstring, isregex=True):
+        """Replace all the matches in the selection"""
+        sel_s = min(stc.GetSelection())
+        stc.BeginUndoAction()
+        for match in reversed(matches):
+            start, end = match.span()
+            start += sel_s
+            end += sel_s
+            if isregex:
+                value = match.expand(rstring.encode('utf-8')).decode('utf-8')
+            else:
+                value = rstring
+            stc.SetTargetStart(start)
+            stc.SetTargetEnd(end)
+            stc.ReplaceTarget(value)
+        stc.EndUndoAction()
+
     def SetFileFilters(self, filters):
         """Set the file filter to use
         @param filters: string '*.py *.pyw'
@@ -627,27 +736,6 @@ class SearchController(object):
 
 #-----------------------------------------------------------------------------#
 
-def FormatResult(fname, lnum, match):
-    """Format the search result string
-    @return: string
-    @todo: better unicode handling
-
-    """
-    fname = ed_txt.DecodeString(fname, sys.getfilesystemencoding())
-    if not isinstance(fname, types.UnicodeType):
-        fname = _("DECODING ERROR")
-
-    match = ed_txt.DecodeString(match)
-    if not isinstance(match, types.UnicodeType):
-        match = _("DECODING ERROR")
-    else:
-        match = u" " + match.lstrip()
-
-    rstring = u"%(fname)s (%(lnum)d): %(match)s"
-    return rstring % dict(fname=fname, lnum=lnum+1, match=match)
-
-#-----------------------------------------------------------------------------#
-
 class EdSearchCtrl(wx.SearchCtrl):
     """Creates a simple search control for use in the toolbar
     or a statusbar and the such. Supports incremental search,
