@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 60431 2009-04-28 22:03:48Z CJP $"
-__revision__ = "$Revision: 60431 $"
+__svnid__ = "$Id: ed_stc.py 61142 2009-06-20 05:28:36Z CJP $"
+__revision__ = "$Revision: 61142 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -54,6 +54,30 @@ SPACECHARS = " \t\r\n"
 NONSPACE = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"
 OPERATORS = "./\?[]{}<>!@#$%^&*():=-+\"';,"
 
+#-------------------------------------------------------------------------#
+
+def jumpaction(func):
+    """Decorator method to notify clients about jump actions"""
+    def WrapJump(*args, **kwargs):
+        stc = args[0]
+        pos = stc.GetCurrentPos()
+        line = stc.GetCurrentLine()
+        func(*args, **kwargs)
+        cpos = stc.GetCurrentPos()
+        cline = stc.GetCurrentLine()
+        fname = stc.GetFileName()
+
+        mdata = dict(fname=fname,
+                     prepos=pos, preline=line,
+                     lnum=cline, pos=cpos)
+        tlw = stc.GetTopLevelParent()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_POS_JUMPED, mdata, tlw.GetId()) 
+
+    WrapJump.__name__ = func.__name__
+    WrapJump.__doc__ = func.__doc__
+    return WrapJump
+
+
 #-------------------------------------------------------------------------#
 
 class EditraStc(ed_basestc.EditraBaseStc):
@@ -90,6 +114,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.LOG = wx.GetApp().GetLog()
         self._loading = None
         self.key_handler = KeyHandler(self)
+        self._backup_done = True
+        self._bktimer = wx.Timer(self)
 
         # Macro Attributes
         self._macro = list()
@@ -117,6 +143,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.Bind(wx.EVT_CHAR, self.OnChar)
         self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
         self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
+        self.Bind(wx.EVT_TIMER, self.OnBackupTimer)
 
         # Async file load events
         self.Bind(ed_txt.EVT_FILE_LOAD, self.OnLoadProgress)
@@ -212,23 +239,6 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.EndUndoAction()
 
     #---- Begin Function Definitions ----#
-    def AddLine(self, before=False, indent=False):
-        """Add a new line to the document
-        @keyword before: whether to add the line before current pos or not
-        @keyword indent: autoindent the new line
-        @postcondition: a new line is added to the document
-
-        """
-        if before:
-            self.LineUp()
-
-        self.LineEnd()
-
-        if indent:
-            self.AutoIndent()
-        else:
-            self.InsertText(self.GetCurrentPos(), self.GetEOLChar())
-            self.LineDown()
 
     def Bookmark(self, action):
         """Handles bookmark actions
@@ -261,6 +271,16 @@ class EditraStc(ed_basestc.EditraBaseStc):
         if mark != -1:
             self.GotoLine(mark)
 
+    # TODO: DO NOT use these methods anywhere new they will be removed soon
+    def ShowCommandBar(self):
+        """Open the command bar"""
+        self.GetTopLevelParent().GetEditPane().ShowCommandControl(ed_glob.ID_COMMAND)
+
+    def ShowFindBar(self):
+        """Open the quick-find bar"""
+        self.GetTopLevelParent().GetEditPane().ShowCommandControl(ed_glob.ID_QUICK_FIND)
+    # END TODO
+
     def GetBookmarks(self):
         """Gets a list of all lines containing bookmarks
         @return: list of line numbers
@@ -321,8 +341,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.ToggleAutoIndent(_PGET('AUTO_INDENT'))
         self.ToggleBracketHL(_PGET('BRACKETHL'))
         self.ToggleLineNumbers(_PGET('SHOW_LN'))
-        self.SetViEmulationMode(_PGET('VI_EMU'))
+        self.SetViEmulationMode(_PGET('VI_EMU'), _PGET('VI_NORMAL_DEFAULT'))
         self.SetViewEdgeGuide(_PGET('SHOW_EDGE'))
+        self.EnableAutoBackup(_PGET('AUTOBACKUP'))
         # NOTE: disabled because it is more annoying than it is benificial.
 #        self.SetEndAtLastLine(False)
 
@@ -337,6 +358,19 @@ class EditraStc(ed_basestc.EditraBaseStc):
         else:
             self.LowerCase()
 
+    def EnableAutoBackup(self, enable):
+        """Enable automatic backups
+        @param enable: bool
+
+        """
+        if enable:
+            # TODO: make backup interval configurable
+            if not self._bktimer.IsRunning():
+                self._bktimer.Start(30000) # every 30 seconds
+        else:
+            if self._bktimer.IsRunning():
+                self._bktimer.Stop()
+
     def InvertCase(self):
         """Invert the case of the selected text
         @postcondition: all text in selection has case inverted
@@ -418,6 +452,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             column = linelen
         self.GotoPos(lstart + column)
 
+    @jumpaction
     def GotoLine(self, line):
         """Move caret to begining given line number
         @param line: line to go to (int)
@@ -435,14 +470,33 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetYCaretPolicy(wx.stc.STC_CARET_EVEN, 0)
         self.PostPositionEvent()
 
+    @jumpaction
     def GotoPos(self, pos):
         """Override StyledTextCtrl.GotoPos
-        @param pos: position in buffer to move carat to (int)
+        @param pos: position in buffer to move caret to (int)
 
         """
         super(EditraStc, self).GotoPos(pos)
         self.PostPositionEvent()
 
+    def SetCaretPos(self, pos):
+        """Set the caret position without posting jump events
+        @param pos: position to go to
+
+        """
+        super(EditraStc, self).GotoPos(pos)
+        self.PostPositionEvent()
+
+    def GotoIndentPos(self, line=None):
+        """Move the caret to the end of the indentation
+        on the given line.
+        @param line: line to go to
+
+        """
+        if line is None:
+            line = self.GetCurrentLine()
+        self.GotoPos(self.GetLineIndentPosition(line))
+
     def SetCurrentCol(self, column):
         """Set the current column position on the currently line
         extending the selection.
@@ -457,16 +511,6 @@ class EditraStc(ed_basestc.EditraBaseStc):
             column = linelen
         self.SetCurrentPos(lstart + column)
 
-    def GotoIndentPos(self, line=None):
-        """Move the caret to the end of the indentation
-        on the given line.
-        @param line: line to go to
-
-        """
-        if line is None:
-            line = self.GetCurrentLine()
-        self.GotoPos(self.GetLineIndentPosition(line))
-
     def DeleteForward(self):
         """Delete the selection, or if there is no selection, then
         delete the character to the right of the cursor.
@@ -490,6 +534,37 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         return self._config['autocomp']
 
+    def OnBackupTimer(self, evt):
+        """Backup the buffer to a backup file.
+        @param evt: wx.TimerEvent
+
+        """
+        fname = self.GetFileName()
+        # If the file is loading or is over 5MB don't do automatic backups.
+        if self.IsLoading() or ebmlib.GetFileSize(fname) > 5242880:
+            return
+
+        # If the file is different than the last save point make the backup.
+        bkupmgr = ebmlib.FileBackupMgr(None, u"%s.edbkup")
+        if not self._backup_done and \
+           (not bkupmgr.HasBackup(fname) or bkupmgr.IsBackupNewer(fname)):
+            writer = bkupmgr.GetBackupWriter(self.File)
+            try:
+                writer(self.GetText())
+            except:
+                return
+            msg = _("File backup performed: %s") % fname
+            nevt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.GetId(),
+                                        msg, ed_glob.SB_INFO)
+            wx.PostEvent(self.GetTopLevelParent(), nevt)
+            self._backup_done = True
+
+    def OnModified(self, evt):
+        """Overrides base modified handler"""
+        super(EditraStc, self).OnModified(evt)
+        if not self.IsLoading():
+            self._backup_done = False
+
     def OnKeyDown(self, evt):
         """Handles keydown events, currently only deals with
         auto indentation.
@@ -602,16 +677,21 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         evt.Skip()
         self.PostPositionEvent()
+        tlw = self.GetTopLevelParent()
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_KEYUP,
-                           (evt.GetPositionTuple(), evt.GetKeyCode()))
+                           (evt.GetPositionTuple(), evt.GetKeyCode()),
+                           tlw.GetId())
 
     def PostPositionEvent(self):
         """Post an event to update the status of the line/column"""
-        pos = self.GetPos()
-        msg = _("Line: %(lnum)d  Column: %(cnum)d") % dict(lnum=pos[0], cnum=pos[1])
+        line, column = self.GetPos()
+        pinfo = dict(lnum=line, cnum=column)
+        msg = _("Line: %(lnum)d  Column: %(cnum)d") % pinfo
         nevt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.GetId(),
                                     msg, ed_glob.SB_ROWCOL)
-        wx.PostEvent(self.GetTopLevelParent(), nevt)
+        tlw = self.GetTopLevelParent()
+        wx.PostEvent(tlw, nevt)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_POS_CHANGED, pinfo, tlw.GetId())
 
     def OnRecordMacro(self, evt):
         """Records macro events
@@ -931,9 +1011,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         elif e_id == ed_glob.ID_WORD_WRAP:
             self.SetWrapMode(not self.GetWrapMode())
         elif e_id == ed_glob.ID_JOIN_LINES:
-            self.SetTargetStart(self.GetSelectionStart())
-            self.SetTargetEnd(self.GetSelectionEnd())
-            self.LinesJoin()
+            self.LinesJoinSelected()
         elif e_id == ed_glob.ID_INDENT_GUIDES:
             self.SetIndentationGuides(not bool(self.GetIndentationGuides()))
         elif e_id == ed_glob.ID_HLCARET_LINE:
@@ -1156,6 +1234,15 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetTargetEnd(self.GetLineEndPosition(eline))
         self.ReplaceTarget(u' '.join(lines))
 
+    def LinesJoinSelected(self):
+        """Similar to LinesJoin, but operates on selection
+        @see: LinesJoin
+
+        """
+        self.SetTargetStart(self.GetSelectionStart())
+        self.SetTargetEnd(self.GetSelectionEnd())
+        self.LinesJoin()
+
     def LineMoveUp(self):
         """Move the current line up"""
         linenum = self.GetCurrentLine()
@@ -1210,15 +1297,17 @@ class EditraStc(ed_basestc.EditraBaseStc):
         mode = mode_map.get(mode, wx.stc.STC_EOL_LF)
         super(EditraStc, self).SetEOLMode(mode)
 
-    def SetViEmulationMode(self, use_vi):
+    def SetViEmulationMode(self, use_vi, use_normal=False):
         """Activate/Deactivate Vi eumulation mode
         @param use_vi: Turn vi emulation on/off
         @type use_vi: boolean
+        @keyword use_normal: Start in normal mode
+        @type use_normal: boolean
 
         """
         self.key_handler.ClearMode()
         if use_vi:
-            self.key_handler = ViKeyHandler(self)
+            self.key_handler = ViKeyHandler(self, use_normal)
         else:
             self.key_handler = KeyHandler(self)
 
@@ -1381,123 +1470,6 @@ class EditraStc(ed_basestc.EditraBaseStc):
             lineNum = self.GetCurrentLine()
         super(EditraStc, self).ToggleFold(lineNum)
 
-    def MoveCurrentPos(self, offset, extend=False):
-        """Move caret by the given offset
-        @note: only use it for movement within a line
-
-        """
-        if offset > 0:
-            step = self.CharRight #XXX breaks RTL, but is RTL supported?
-            if extend:
-                step = self.CharRightExtend
-        else:
-            step = self.CharLeft
-            if extend:
-                step = self.CharLeftExtend
-
-        #XXX is there a better reliable way than repeating basic movements?
-        repeat = abs(offset)
-        for i in range(repeat):
-            step()
-
-    def _FindChar(self, char, repeat=1, reverse=False, extra_offset=0):
-        """Find the position of the next (ith) 'char' character
-        on the current line
-
-        @note used by vim motions for finding a character on a line (f,F,t,T)
-        @param char: the char to be found
-        @keyword repeat: how many times to repeat the serach
-        @keyword reverse: whether to search backwards
-        @keyword extra_offset: extra offset to be applied to the return value
-
-        @return: offset from caret position
-
-        """
-        text, pos = self.GetCurLine()
-        oldpos = pos
-        if not reverse:
-            for i in range(repeat):
-                pos = text.find(char, pos+1) # pos is on the char itself
-                if pos == -1:
-                    break # -1 + 1 = 0
-        else:
-            for i in range(repeat):
-                pos = text.rfind(char, 0, pos)
-                if pos == -1:
-                    break # TEST not sure but we need this as well
-
-        # if pos == -1 then the char was not found
-        # but offset could be arbitrary
-        # (although should never be other than 0 or -1) so do a bound check
-        newpos = pos + extra_offset
-        if newpos not in range(len(text)):
-            return 0
-
-        return newpos - oldpos;
-
-    def FindNextChar(self, char, repeat=1, extend=False):
-        """Move caret to the next (ith) occurance of char on the current line
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindPrevChar(self, char, repeat=1, extend=False):
-        """Move caret to the previous (ith) occurance of char on the current line
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-
-        offset = self._FindChar(char, repeat, reverse=True)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindTillNextChar(self, char, repeat=1, extend=False):
-        """Move caret until right before the next ith occurance of char
-
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat, extra_offset=-1)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindTillPrevChar(self, char, repeat=1, extend=False):
-        """Move caret until right before the previous ith occurance of char
-
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat, reverse=True, extra_offset=-1)
-        self.MoveCurrentPos(offset, extend)
-
-
-    def GetIdentifierUnderCursor(self):
-        """Returns the identifier under the cursor (if any)"""
-        line, pos = self.GetCurLine()
-        while pos > 0: # rewind to first char of identifier
-            c = line[pos-1]
-            if c == '_' or c.isalnum():
-               pos = pos - 1
-            else:
-                break
-
-        start = pos
-        while pos < len(line): # find end of identifier
-            c = line[pos]
-            if c == '_' or c.isalnum():
-               pos = pos + 1
-            else:
-                break
-        return line[start:pos]
-
     def WordLeft(self):
         """Move caret to begining of previous word
         @note: override builtin to include extra characters in word
@@ -1666,7 +1638,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 return False, msg
             else:
                 self.GotoPos(cpos)
-                ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.GetFileName())
+                context = self.GetTopLevelParent().GetId()
+                ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED,
+                                   self.GetFileName(), context)
                 return True, ''
         else:
             self.LOG("[ed_stc][err] %s does not exists, cant reload." % cfile)
@@ -1706,8 +1680,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         result = True
         try:
+            tlw_id = self.GetTopLevelParent().GetId()
             ed_msg.PostMessage(ed_msg.EDMSG_FILE_SAVE,
-                               (path, self.GetLangId()))
+                               (path, self.GetLangId()), tlw_id)
             self.File.SetPath(path)
             self.LOG("[ed_stc][info] Writing file %s, with encoding %s" % \
                      (path, self.File.GetEncoding()))
@@ -1734,8 +1709,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.SetFileName(path)
 
         wx.CallAfter(ed_msg.PostMessage,
-                    ed_msg.EDMSG_FILE_SAVED,
-                    (path, self.GetLangId()))
+                     ed_msg.EDMSG_FILE_SAVED,
+                     (path, self.GetLangId()),
+                     tlw_id)
 
         return result
 
@@ -1747,6 +1723,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
         super(EditraStc, self).ConfigureLexer(file_ext)
 
         # Notify that lexer has changed
+        self.LOG("[ed_stc][info] Lexer change notification for context %d" %
+                 self.GetTopLevelParent().GetId())
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_LEXER,
-                           (self.GetFileName(), self.GetLangId()))
+                           (self.GetFileName(), self.GetLangId()),
+                           context=self.GetTopLevelParent().GetId())
         return True
