@@ -12,8 +12,8 @@ Text/Unicode handling functions and File wrapper class
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_txt.py 60465 2009-05-02 00:16:48Z CJP $"
-__revision__ = "$Revision: 60465 $"
+__svnid__ = "$Id: ed_txt.py 61040 2009-06-13 08:16:04Z CJP $"
+__revision__ = "$Revision: 61040 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -78,7 +78,8 @@ class EdFile(ebmlib.FileObjectImpl):
     """
     def __init__(self, path=u'', modtime=0):
         """Create the file wrapper object
-        @param path: the absolute path to the file
+        @keyword path: the absolute path to the file
+        @keyword modtime: file modification time
 
         """
         ebmlib.FileObjectImpl.__init__(self, path, modtime)
@@ -100,6 +101,20 @@ class EdFile(ebmlib.FileObjectImpl):
         """Cleanup callback"""
         pass
 
+    def Clone(self):
+        """Clone the file object
+        @return: EdFile
+
+        """
+        fileobj = EdFile(self.GetPath(), self.GetModtime())
+        fileobj.SetLastError(self.last_err)
+        fileobj.SetEncoding(self.encoding)
+        fileobj.bom = self.bom
+        fileobj._magic = dict(self._magic)
+        for cback in self._mcallback:
+            fileobj.AddModifiedCallback(cback)
+        return fileobj
+
     @property
     def Encoding(self):
         """File encoding property"""
@@ -317,8 +332,6 @@ class EdFile(ebmlib.FileObjectImpl):
         # Check if a magic comment was added or changed
         tbuff = StringIO(value)
         enc = CheckMagicComment([ tbuff.readline() for x in range(2) ])
-        tbuff.close()
-        del tbuff
 
         # Update encoding if necessary
         if enc is not None:
@@ -330,8 +343,11 @@ class EdFile(ebmlib.FileObjectImpl):
             writer = codecs.getwriter(self.encoding)(self.Handle)
             if self.HasBom():
                 Log("[ed_txt][info] Adding BOM back to text")
-                value = self.bom + value
-            writer.write(value)
+                writer.write(self.bom)
+
+            writer.write(tbuff.getvalue())
+            tbuff.close()
+            del tbuff
             writer.close()
             Log("[ed_txt][info] %s was written successfully" % self.GetPath())
         else:
