@@ -20,8 +20,8 @@ happen during run time.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: profiler.py 60381 2009-04-26 05:53:19Z CJP $"
-__revision__ = "$Revision: 60381 $"
+__svnid__ = "$Id: profiler.py 60682 2009-05-19 00:44:55Z CJP $"
+__revision__ = "$Revision: 60682 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -99,6 +99,7 @@ _DEFAULTS = {
            'USETABS'    : False,            # Use tabs instead of spaces
            'USE_PROXY'  : False,            # Use Proxy server settings?
            'VI_EMU'     : False,            # Use Vi emulation mode
+           'VI_NORMAL_DEFAULT' : False,     # Use Normal mode by default
            'WARN_EOL'   : True,             # Warn about mixed eol characters
            'WRAP'       : False,            # Use Wordwrap
            'WSIZE'      : (700, 450)        # Mainwindow size
@@ -232,12 +233,11 @@ class Profile(dict):
             # Only write if given an absolute path
             if not os.path.isabs(path):
                 return False
-
             self.Set('MYPROFILE', path)
-            UpdateProfileLoader()
             fhandle = open(path, 'wb')
             cPickle.dump(self.copy(), fhandle, cPickle.HIGHEST_PROTOCOL)
             fhandle.close()
+            UpdateProfileLoader()
         except (IOError, cPickle.PickleError), msg:
             dev_tool.DEBUGP(u"[profile][err] %s" % msg)
             return False
@@ -457,6 +457,7 @@ def ProfileVersionStr():
 
 def UpdateProfileLoader():
     """Updates Loader File
+    @precondition: MYPROFILE has been set
     @postcondition: on disk profile loader is updated
     @return: 0 if no error, non zero for error condition
 
@@ -466,7 +467,7 @@ def UpdateProfileLoader():
         return 1
 
     prof_name = Profile_Get('MYPROFILE')
-    if not prof_name:
+    if not prof_name or not os.path.isabs(prof_name):
         prof_name = CONFIG['PROFILE_DIR'] + 'default.ppb'
 
     if not os.path.exists(prof_name):
