@@ -18,8 +18,8 @@ AUTHOR: Cody Precord
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: synglob.py 56840 2008-11-18 19:32:54Z CJP $"
-__revision__ = "$Revision: 56840 $"
+__svnid__ = "$Id: synglob.py 60800 2009-05-30 01:29:53Z CJP $"
+__revision__ = "$Revision: 60800 $"
 
 #-----------------------------------------------------------------------------#
 # Dependencies
@@ -33,7 +33,8 @@ from synextreg import *
 #-----------------------------------------------------------------------------#
 
 # Maps file types to syntax definitions
-LANG_MAP = {LANG_68K    : (ID_LANG_68K,    stc.STC_LEX_ASM,      'asm68k'),
+LANG_MAP = {LANG_4GL    : (ID_LANG_4GL,    stc.STC_LEX_SQL,      'progress'),
+            LANG_68K    : (ID_LANG_68K,    stc.STC_LEX_ASM,      'asm68k'),
             LANG_ADA    : (ID_LANG_ADA,    stc.STC_LEX_ADA,      'ada'),
             LANG_APACHE : (ID_LANG_APACHE, stc.STC_LEX_CONF,     'apache'),
             LANG_AS     : (ID_LANG_AS,     stc.STC_LEX_CPP,      'actionscript'),
@@ -42,6 +43,7 @@ LANG_MAP = {LANG_68K    : (ID_LANG_68K,    stc.STC_LEX_ASM,      'asm68k'),
             LANG_BOO    : (ID_LANG_BOO,    stc.STC_LEX_PYTHON,   'boo'),
             LANG_C      : (ID_LANG_C,      stc.STC_LEX_CPP,      'cpp'),
             LANG_CAML   : (ID_LANG_CAML,   stc.STC_LEX_CAML,     'caml'),
+            LANG_COBRA  : (ID_LANG_COBRA,  stc.STC_LEX_PYTHON,   'cobra'),
             LANG_COLDFUSION : (ID_LANG_COLDFUSION, stc.STC_LEX_HTML, 'html'),
             LANG_CPP    : (ID_LANG_CPP,    stc.STC_LEX_CPP,      'cpp'),
             LANG_CSH    : (ID_LANG_CSH,    stc.STC_LEX_BASH,     'sh'),
@@ -64,6 +66,7 @@ LANG_MAP = {LANG_68K    : (ID_LANG_68K,    stc.STC_LEX_ASM,      'asm68k'),
             LANG_HAXE   : (ID_LANG_HAXE, stc.STC_LEX_CPP,        'haxe'),
             LANG_HTML   : (ID_LANG_HTML,   stc.STC_LEX_HTML,     'html'),
             LANG_INNO   : (ID_LANG_INNO,   stc.STC_LEX_INNOSETUP, 'inno'),
+            LANG_ISSL   : (ID_LANG_ISSL,   stc.STC_LEX_CONTAINER, 'issuelist'),
             LANG_JAVA   : (ID_LANG_JAVA,   stc.STC_LEX_CPP,      'java'),
             LANG_JS     : (ID_LANG_JS,     stc.STC_LEX_CPP,      'javascript'),
             LANG_KIX    : (ID_LANG_KIX,    stc.STC_LEX_KIX,      'kix'),
@@ -106,7 +109,9 @@ LANG_MAP = {LANG_68K    : (ID_LANG_68K,    stc.STC_LEX_ASM,      'asm68k'),
             LANG_VERILOG: (ID_LANG_VERILOG, stc.STC_LEX_VERILOG, 'verilog'),
             LANG_VHDL   : (ID_LANG_VHDL,   stc.STC_LEX_VHDL,     'vhdl'),
             LANG_XML    : (ID_LANG_XML,    stc.STC_LEX_XML,      'xml'),
-            LANG_YAML   : (ID_LANG_YAML,   stc.STC_LEX_YAML,     'yaml')
+            LANG_YAML   : (ID_LANG_YAML,   stc.STC_LEX_YAML,     'yaml'),
+            LANG_GROOVY : (ID_LANG_GROOVY,   stc.STC_LEX_CPP,    'groovy'),
+            LANG_XTEXT  : (ID_LANG_XTEXT,   stc.STC_LEX_CONTAINER,'xtext')
             }
 
 # Dynamically finds the language description string that matches the given
