@@ -12,8 +12,8 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 60484 2009-05-02 18:53:43Z CJP $"
-__revision__ = "$Revision: 60484 $"
+__svnid__ = "$Id: util.py 61179 2009-06-23 22:47:20Z CJP $"
+__revision__ = "$Revision: 61179 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -233,6 +233,39 @@ class DropTargetFT(wx.PyDropTarget):
 
 #---- End FileDropTarget ----#
 
+class EdClipboard(ebmlib.CycleCache):
+    def GetNext(self):
+        """Get the next item in the cache"""
+        # Initialize the clipboard if it hasn't been loaded yet and
+        # there is something in the system clipboard
+        if self.GetCurrentSize() == 0:
+            txt = GetClipboardText()
+            if txt is not None:
+                self.Put(txt)
+
+        return super(EdClipboard, self).GetNext()
+
+    def IsAtIndex(self, txt):
+        """Is the passed in phrase at the current cycle index in the
+        cache. Used to check if index should be reset or to continue in
+        the cycle.
+        @param txt: selected text
+
+        """
+        pre = self.PeekPrev()
+        next = self.PeekNext()
+        if txt in (pre, next):
+            return True
+        else:
+            return False
+
+    def Put(self, txt):
+        """Put some text in the clipboard"""
+        pre = self.PeekPrev()
+        next = self.PeekNext()
+        if len(txt) and txt not in (pre, next):
+            self.PutItem(txt)
+
 #---- Misc Common Function Library ----#
 # Used for holding the primary selection on mac/msw
 FAKE_CLIPBOARD = None
@@ -373,13 +406,6 @@ def GetFileWriter(file_name, enc='utf-8'):
         writer = file_h
     return writer
 
-def GetPathName(path):
-    """Gets the path minus filename
-    @param path: full path to get base of
-
-    """
-    return os.path.split(path)[0]
-
 def GetFileManagerCmd():
     """Get the file manager open command for the current os. Under linux
     it will check for xdg-open, nautilus, konqueror, and Thunar, it will then
@@ -401,21 +427,6 @@ def GetFileManagerCmd():
         else:
             return 'nautilus'
 
-def GetFileName(path):
-    """Gets last atom on end of string as filename
-    @param path: full path to get filename from
-
-    """
-    return os.path.split(path)[-1]
-
-def GetExtension(file_str):
-    """Gets last atom at end of string as extension if
-    no extension whole string is returned
-    @param file_str: path or file name to get extension from
-
-    """
-    return file_str.split('.')[-1]
-
 def HasConfigDir(loc=u""):
     """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
@@ -484,9 +495,9 @@ def CreateConfigDir():
             MakeConfigDir(cfg)
 
     import profiler
-    profiler.Profile().LoadDefaults()
+    profiler.TheProfile.LoadDefaults()
     profiler.Profile_Set("MYPROFILE", dest_file)
-    profiler.Profile().Write(dest_file)
+    profiler.TheProfile.Write(dest_file)
     profiler.UpdateProfileLoader()
 
 def ResolvConfigDir(config_dir, sys_only=False):
